package de.intevation.flys.exports;

import org.apache.log4j.Logger;

import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

import de.intevation.artifacts.Artifact;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.WQCKms;
import de.intevation.flys.artifacts.model.WQKms;


/**
 * An OutGenerator that generates discharge longitudinal section curves.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class DischargeLongitudinalSectionGenerator
extends      LongitudinalSectionGenerator
{
    private static Logger logger =
        Logger.getLogger(DischargeLongitudinalSectionGenerator.class);



    public DischargeLongitudinalSectionGenerator() {
        super();
    }


    @Override
    public void doOut(
        Artifact artifact,
        Facet    facet,
        Document attr,
        boolean  visible
    ) {
        logger.debug("DischargeLongitudinalSectionGenerator.doOut");

        if (facet == null) {
            return;
        }

        String name = facet.getName();

        if (name == null) {
            return;
        }

        FLYSArtifact flys = (FLYSArtifact) artifact;
        Facet        f    = flys.getNativeFacet(facet);

        if (name.equals(DISCHARGE_LONGITUDINAL_W)) {
            doWOut((WQKms) f.getData(artifact, context), facet, attr, visible);
        }
        else if (name.equals(DISCHARGE_LONGITUDINAL_Q)) {
            doQOut((WQKms) f.getData(artifact, context), facet, attr, visible);
        }
        else if (name.equals(DISCHARGE_LONGITUDINAL_C)) {
            doCorrectedWOut(
                (WQCKms) f.getData(artifact, context),
                facet,
                attr,
                visible);
        }
        else {
            logger.warn("Unknown facet name: " + name);
        }
    }


    /**
     * Adds a new series for the corrected W curve.
     *
     * @param wqckms The object that contains the corrected W values.
     * @param theme The theme that contains styling information.
     */
    protected void doCorrectedWOut(
        WQCKms   wqckms,
        Facet    facet,
        Document theme,
        boolean  visible
    ) {
        logger.debug("DischargeLongitudinalSectionGenerator.doCorrectedWOut");

        int size = wqckms.size();

        if (size > 0) {
            XYSeries series = new StyledXYSeries(
                facet.getDescription(),
                theme);

            for (int i = 0; i < size; i++) {
                series.add(wqckms.getKm(i), wqckms.getC(i));
            }

            addFirstAxisSeries(series, visible);
        }

        if (wqckms.guessWaterIncreasing()) {
            setInverted(true);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
